@@ -1,4 +1,4 @@
-# Copyright (c) [2017-2020] SUSE LLC
+# Copyright (c) [2017-2021] SUSE LLC
 #
 # All Rights Reserved.
 #
@@ -34,6 +34,7 @@ module Y2Partitioner
     module FstabCommon
       # @param controller [Y2Partitioner::Actions::Controllers:Filesystem]
       def initialize(controller)
+        super()
         textdomain "storage"
 
         @controller = controller
@@ -82,7 +83,7 @@ module Y2Partitioner
       # Removes from the MountPoint object the options that not longer apply
       def delete_fstab_option!(option)
         # The options can only be modified using MountPoint#mount_options=
-        mount_point.mount_options = mount_point.mount_options.reject { |o| o =~ option }
+        mount_point.mount_options = mount_point.mount_options.grep_v(option)
       end
 
       # Adds new options to the MountPoint object
@@ -173,6 +174,7 @@ module Y2Partitioner
       SUPPORTED_FILESYSTEMS = [:btrfs, :ext2, :ext3, :ext4].freeze
 
       def initialize(controller)
+        super # to pass controller to FstabCommon
         @controller = controller
 
         self.handle_all_events = true
@@ -186,20 +188,12 @@ module Y2Partitioner
         disable if !supported_by_filesystem?
       end
 
-      # @macro seeAbstractWidget
-      def handle(event)
-        case event["ID"]
-        when :help
-          help = []
-
-          widgets.each do |w|
-            help << w.help if w.respond_to? "help"
-          end
-
-          Yast::Wizard.ShowHelp(help.join("\n"))
-        end
-
-        nil
+      def help
+        # FIXME: this method causes duplicated help text
+        # if Wizard.SetHelpText is used
+        Yast::CWM.widgets_in_contents(contents).find_all do |w|
+          w.respond_to?(:help)
+        end.map(&:help).join("\n")
       end
 
       # @macro seeCustomWidget
@@ -208,8 +202,7 @@ module Y2Partitioner
           VBox(
             Left(MountBy.new(@controller)),
             VSpacing(1),
-            Left(VolumeLabel.new(@controller, self)),
-            VSpacing(1),
+            * ui_term_with_vspace(VolumeLabel.new(@controller, self)),
             Left(GeneralOptions.new(@controller)),
             Left(FilesystemsOptions.new(@controller)),
             * ui_term_with_vspace(JournalOptions.new(@controller)),
@@ -256,6 +249,13 @@ module Y2Partitioner
         @parent_widget = parent_widget
       end
 
+      # @see FstabCommon::supported_by_filesystem?
+      #
+      # @return [Boolean]
+      def supported_by_filesystem?
+        filesystem.supports_label?
+      end
+
       # @macro seeAbstractWidget
       def label
         _("Volume &Label")
@@ -271,6 +271,7 @@ module Y2Partitioner
         self.value = filesystem.label
         Yast::UI.ChangeWidget(Id(widget_id), :ValidChars, valid_chars)
         Yast::UI.ChangeWidget(Id(widget_id), :InputMaxLength, input_max_length)
+        disable unless can_set_volume_label?
       end
 
       # Validates uniqueness of the given label. The presence of the label is also
@@ -288,6 +289,27 @@ module Y2Partitioner
       # @return [Widgets::FstabOptions]
       attr_reader :parent_widget
 
+      # Check if the volume label can be set.
+      #
+      # @return [Boolean]
+      def can_set_volume_label?
+        return true unless @controller.mounted_in_system_graph?
+
+        blk_dev = @controller.blk_device_name
+        fs_type = @controller.filesystem_type
+        log.info("#{blk_dev} type #{fs_type} is mounted")
+        # Can't change the volume label for a mounted Btrfs or swap (bsc#1211337)
+        !btrfs? && !swap?
+      end
+
+      def btrfs?
+        @controller.filesystem_type.is?(:btrfs)
+      end
+
+      def swap?
+        @controller.filesystem_type.is?(:swap)
+      end
+
       # Checks whether a label is given when the filesystem is mounted by label
       #
       # @note An error popup is presented when the filesystem is mounted by label
@@ -493,8 +515,8 @@ module Y2Partitioner
       # @macro seeAbstractWidget
       def help
         _("<p><b>Mount Read-Only:</b>\n" \
-        "Writing to the file system is not possible. Default is false. During installation\n" \
-        "the file system is always mounted read-write.</p>")
+          "Writing to the file system is not possible. Default is false. During installation\n" \
+          "the file system is always mounted read-write.</p>")
       end
     end
 
@@ -512,7 +534,7 @@ module Y2Partitioner
       # @macro seeAbstractWidget
       def help
         _("<p><b>Mountable by User:</b>\nThe file system may be " \
-        "mounted by an ordinary user. Default is false.</p>\n")
+          "mounted by an ordinary user. Default is false.</p>\n")
       end
     end
 
@@ -664,12 +686,12 @@ module Y2Partitioner
       # @macro seeAbstractWidget
       def help
         _("<p><b>Data Journaling Mode:</b>\n" \
-        "Specifies the journaling mode for file data.\n" \
-        "<tt>journal</tt> -- All data is committed to the journal prior to being\n" \
-        "written into the main file system. Highest performance impact.<br>\n" \
-        "<tt>ordered</tt> -- All data is forced directly out to the main file system\n" \
-        "prior to its metadata being committed to the journal. Medium performance impact.<br>\n" \
-        "<tt>writeback</tt> -- Data ordering is not preserved. No performance impact.</p>\n")
+          "Specifies the journaling mode for file data.\n" \
+          "<tt>journal</tt> -- All data is committed to the journal prior to being\n" \
+          "written into the main file system. Highest performance impact.<br>\n" \
+          "<tt>ordered</tt> -- All data is forced directly out to the main file system\n" \
+          "prior to its metadata being committed to the journal. Medium performance impact.<br>\n" \
+          "<tt>writeback</tt> -- Data ordering is not preserved. No performance impact.</p>\n")
       end
     end
 
@@ -680,6 +702,7 @@ module Y2Partitioner
       include FstabCommon
 
       def initialize(controller, parent_widget)
+        super(controller) # pass controller to FstabCommon
         textdomain "storage"
         @controller = controller
         @parent_widget = parent_widget
@@ -722,10 +745,14 @@ module Y2Partitioner
 
       # @macro seeAbstractWidget
       def help
-        _("<p><b>Arbitrary Option Value:</b> " \
+        _(
+          "<p><b>Arbitrary Option Value:</b> " \
           "Enter any other mount options here, separated with commas. " \
-          "Notice that this does not do any checking, so be careful " \
-          "what you enter here!</p>")
+          "Notice that this does not do any checking, so be careful what you enter here!<br>" \
+          "If you want to have utf8 encoded filenames with limited case-insensitivity then " \
+          "add 'utf8' here and leave the Charset for File Names field empty." \
+          "</p>"
+        )
       end
 
       private
@@ -759,7 +786,7 @@ module Y2Partitioner
       # Return all values that are handled by other widgets in this widget tree.
       # @return [Array<String>]
       def other_values
-        return [] unless @parent_widget&.respond_to?(:values)
+        return [] unless @parent_widget.respond_to?(:values)
 
         @other_values ||= @parent_widget.values
       end
@@ -767,7 +794,7 @@ module Y2Partitioner
       # Return all regexps that are handled by other widgets in this widget tree.
       # @return [Array<Regexp>]
       def other_regexps
-        return [] unless @parent_widget&.respond_to?(:regexps)
+        return [] unless @parent_widget.respond_to?(:regexps)
 
         @other_regexps ||= @parent_widget.regexps
       end
@@ -787,6 +814,7 @@ module Y2Partitioner
       def widgets
         [
           SwapPriority.new(@controller),
+          TmpfsSize.new(@controller),
           IOCharset.new(@controller),
           Codepage.new(@controller)
         ]
@@ -818,7 +846,56 @@ module Y2Partitioner
       # @macro seeAbstractWidget
       def help
         _("<p><b>Swap Priority:</b>\nEnter the swap priority. " \
-        "Higher numbers mean higher priority.</p>\n")
+          "Higher numbers mean higher priority.</p>\n")
+      end
+    end
+
+    # Size for tmpfs
+    class TmpfsSize < CWM::InputField
+      include FstabCommon
+
+      # Possible values of the widget
+      VALUES = ["size="].freeze
+      # Format of the option
+      REGEXP  = /^size=/
+      # Default value of the widget
+      DEFAULT = "".freeze
+      # Regular expression to validate the content of #value
+      VALUE_REGEXP = /^\d+(k|m|g|%)?$/i
+
+      # @macro seeAbstractWidget
+      def label
+        _("Max Size")
+      end
+
+      # @macro seeAbstractWidget
+      def store
+        delete_fstab_option!(REGEXP)
+        add_fstab_option("size=#{value}") if value && !value.empty?
+      end
+
+      # @macro seeAbstractWidget
+      def validate
+        return true if value.nil? || value.empty?
+        return true if value =~ VALUE_REGEXP
+
+        Yast::Popup.Error(
+          _(
+            "The size must be a number,\n" \
+            "optionally followed by a unit (k, m or g) or by a percent sign (%)."
+          )
+        )
+        focus
+        false
+      end
+
+      # @macro seeAbstractWidget
+      def help
+        _("<p><b>Max Size:</b>\nUpper limit on the size of the file system. " \
+          "The size is given in bytes by default, optionally followed by a <b>k</b>, <b>m</b> " \
+          "or <b>g</b> suffix to specify KiB, MiB or GiB. The size may also have a <b>% </b> " \
+          "suffix to limit this instance to a percentage of physical RAM. " \
+          "The default, when neither this option nor nr_blocks is specified, is 50%.</p>\n")
       end
     end
 
@@ -840,16 +917,18 @@ module Y2Partitioner
       end
 
       def default_value
-        iocharset = filesystem.type.iocharset
-        ITEMS.include?(iocharset) ? iocharset : ITEMS.first
+        ITEMS.first
       end
 
       def supported_by_mount_path?
         return false if mount_path.nil?
 
-        # "iocharset=utf8" breaks VFAT case insensitivity (bsc#1080731).
-        # See also boot_fstab_options() in lib/y2storage/filesystems/type.rb
-        mount_path != "/boot" && !mount_path.start_with?("/boot/")
+        # We used to disable the iocharset dialog for filesystems used for booting since
+        # iocharset=utf8 breaks VFAT case insensitivity (bsc#1080731).
+        #
+        # That's taken care of now in {Filesystems::Type#patch_iocharset}
+        # when we create a new file system.
+        true
       end
 
       # @macro seeAbstractWidget
@@ -859,8 +938,15 @@ module Y2Partitioner
 
       # @macro seeAbstractWidget
       def help
-        _("<p><b>Charset for File Names:</b>\nSet the charset used for display " \
-        "of file names in Windows partitions.</p>\n")
+        _(
+          "<p><b>Charset for File Names:</b>\n" \
+          "Set the charset used for display of file names in Windows partitions.<br>\n" \
+          "Note that choosing 'utf8' here will implicitly make " \
+          "filename comparison case-sensitve.<br>\n" \
+          "To avoid this, leave this field empty and instead add 'utf8' " \
+          "to the Arbitrary Option Value field." \
+          "</p>\n"
+        )
       end
     end
 
@@ -872,8 +958,7 @@ module Y2Partitioner
       ITEMS = ["", "437", "852", "932", "936", "949", "950"].freeze
 
       def default_value
-        cp = filesystem.type.codepage
-        ITEMS.include?(cp) ? cp : ITEMS.first
+        ITEMS.first
       end
 
       # @macro seeAbstractWidget
@@ -884,7 +969,7 @@ module Y2Partitioner
       # @macro seeAbstractWidget
       def help
         _("<p><b>Codepage for Short FAT Names:</b>\nThis codepage is used for " \
-        "converting to shortname characters on FAT file systems.</p>\n")
+          "converting to shortname characters on FAT file systems.</p>\n")
       end
     end
   end
