@@ -30,7 +30,6 @@ require "y2storage/filesystems/base"
 require "y2storage/filesystems/blk_filesystem"
 require "y2storage/filesystems/tmpfs"
 require "y2storage/filesystems/nfs"
-require "y2storage/filesystems/tmpfs"
 require "y2storage/lvm_lv"
 require "y2storage/lvm_vg"
 require "y2storage/md"
@@ -39,6 +38,7 @@ require "y2storage/partition"
 require "y2storage/storage_class_wrapper"
 require "y2storage/storage_manager"
 require "y2storage/storage_features_list"
+require "y2storage/issues_manager"
 
 module Y2Storage
   # The master container of libstorage.
@@ -48,7 +48,7 @@ module Y2Storage
   # the inspected system) or a possible target state.
   #
   # This is a wrapper for Storage::Devicegraph
-  class Devicegraph
+  class Devicegraph # rubocop:disable Metrics/ClassLength
     include Yast::Logger
     include StorageClassWrapper
     wrap_class Storage::Devicegraph
@@ -100,11 +100,12 @@ module Y2Storage
     storage_forward :storage_used_features, to: :used_features
     private :storage_used_features
 
-    # @!method copy(dest)
+    # @!method storage_copy(dest)
     #   Copies content to another devicegraph
     #
     #   @param dest [Devicegraph] destination devicegraph
-    storage_forward :copy
+    storage_forward :storage_copy, to: :copy
+    private :storage_copy
 
     # @!method find_device(device)
     #   Find a device by its {Device#sid sid}
@@ -140,17 +141,21 @@ module Y2Storage
       new(devicegraph)
     end
 
+    # @return [IssuesManager]
+    def issues_manager
+      @issues_manager ||= IssuesManager.new(self)
+    end
+
     # @return [Devicegraph]
     def dup
       new_graph = ::Storage::Devicegraph.new(to_storage_value.storage)
-      copy(new_graph)
-      Devicegraph.new(new_graph)
+      copy(Devicegraph.new(new_graph))
     end
     alias_method :duplicate, :dup
 
     # Copies the devicegraph into another one, but avoiding to copy into itself
     #
-    # @return[Boolean] true if the devicegraph was copied; false otherwise.
+    # @return [Boolean] true if the devicegraph was copied; false otherwise.
     def safe_copy(devicegraph)
       # Never try to copy into itself. Bug#1069671
       return false if devicegraph.equal?(self)
@@ -159,6 +164,15 @@ module Y2Storage
       true
     end
 
+    # Copies the devicegraph into another one
+    #
+    # @param devicegraph [Devicegraph]
+    def copy(devicegraph)
+      storage_copy(devicegraph)
+      devicegraph.issues_manager.probing_issues = issues_manager.probing_issues
+      devicegraph
+    end
+
     # Checks the devicegraph and logs the errors
     #
     # Note that the errors reported as exception are logged too.
@@ -303,6 +317,13 @@ module Y2Storage
       Filesystems::Base.all(self)
     end
 
+    # All mount points in the devicegraph, in no particular order
+    #
+    # @return [Array<MountPoint>]
+    def mount_points
+      MountPoint.all(self)
+    end
+
     # @param mountpoint [String] mountpoint of the filesystem (e.g. "/").
     # @return [Boolean]
     def filesystem_in_network?(mountpoint)
@@ -594,6 +615,23 @@ module Y2Storage
       StorageFeaturesList.from_bitfield(storage_used_features(type))
     end
 
+    # List of required (mandatory) storage features used by the devicegraph
+    #
+    # @return [StorageFeaturesList]
+    def required_used_features
+      used_features(required_only: true)
+    end
+
+    # List of optional storage features used by the devicegraph
+    #
+    # @return [StorageFeaturesList]
+    def optional_used_features
+      all = storage_used_features(Storage::UsedFeaturesDependencyType_SUGGESTED)
+      required = storage_used_features(Storage::UsedFeaturesDependencyType_REQUIRED)
+      # Using binary XOR in those bit fields to calculate the difference
+      StorageFeaturesList.from_bitfield(all ^ required)
+    end
+
     private
 
     # Copy of a device tree where hashes have been substituted by sorted
